/* akadáykikerulo_robot2
 * 
 * Akadáylkikerülő robot HC-SR05 ultrahangos szenzorral, melyet 
 * szervo forgat jobbra-balra. Ebben a programban az előre-
 * menetelnél három iránnyba nézünk, hogy szélesebb látókörben
 * észleljük az akadályokat (lásd a look_forward() függvényt!).
 * 
 * Az egyszerű algoritmus szerint előre
 * megy a robot, amíg a magadott határon belül nincs akadály.
 * Akadály eészlelése esetén jobbra-balra néz a robot, s arra megy, 
 * amerre szabad az út. Zsákutca esetén hátrálás, majd újrakezdés 
 * következik.
 */
#include <Servo.h>

Servo servo1;          // Jobbr-balra forgatás
Servo servo2;          // le-fel forgatás

#define B1A 7          // Bal motor előre
#define B1B 6          // Bal motor hátra
#define A1A 5          // Jobb motor előre
#define A1B 4          // Jobb motor hátra
#define trigPin  8     // D8  kivezetés
#define echoPin 11     // D11 kivezetés
#define minDistance 30 // Legkisebb távolság cm-ben
int distance_f;
int distance_r;
int distance_l;

void setup() {
  // Szervók konfigurálása
  servo1.attach(2);           // Szervó1 a D2 kimenetre
  servo1.write(90);           // Középső állásban legyen
  servo2.attach(3);           // Szervó2 a D3 kimenetre
  servo2.write(70);           // Középső állásban legyen
  // Szonár konfigurálás
  pinMode(trigPin, OUTPUT);   // trigPin kimenet
  digitalWrite(trigPin, LOW);
  pinMode(echoPin, INPUT);    // echoPin bemenet
  // Motorvezérlő lábak konfigurálása
  pinMode(B1A, OUTPUT);
  pinMode(B1B, OUTPUT);
  pinMode(A1A, OUTPUT);
  pinMode(A1B, OUTPUT);
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
  delay(5000);
}

void loop() {
  distance_f = look_forward();
  if (distance_f > minDistance) {
    motor_forward();
  } else {
    motor_stop();
    look_around();
    if (distance_r > minDistance) {
      motor_right();
      delay(300);
      motor_stop();
    } else if (distance_l > minDistance) {
      motor_left();
      delay(300);
      motor_stop();
    } else {
      motor_reverse();
      delay(300);
      motor_stop();
    }
  }
}

uint16_t ping(int angle) {
  servo1.write(angle);
  delay(100);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  long duration = pulseIn(echoPin, HIGH);
  uint16_t distance = (duration > 0) ? (duration / 58) : 200;
  return distance;
}

void look_around() {
  servo1.write(50);
  delay(200);
  distance_l = ping(50);
  servo1.write(150);
  delay(200);
  distance_r = ping(160);
}


int look_forward() {
  int temp1 = ping(70);
  int temp2 = ping(90);
  int temp3 = ping(110);
  int temp4 = min(temp1, temp2);
  return min(temp3, temp4);
}

void motor_forward() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_reverse() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_left() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, HIGH);
  digitalWrite(A1A, HIGH);
  digitalWrite(A1B, LOW);
}

void motor_right() {
  digitalWrite(B1A, HIGH);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, HIGH);
}

void motor_stop() {
  digitalWrite(B1A, LOW);
  digitalWrite(B1B, LOW);
  digitalWrite(A1A, LOW);
  digitalWrite(A1B, LOW);
}
